require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.view.*"
import "android.content.Context"
import "cjson"
import "android.os.Environment"

-- Configuration file paths
local BASE_DIR = Environment.getExternalStorageDirectory().getPath().."/MessageManager"
File(BASE_DIR).mkdirs()
local CONFIG_FILE = BASE_DIR.."/config.json"
local MESSAGES_FILE = BASE_DIR.."/messages.json"

-- Data structure
local config = {
  folders = {},
  messages = {}
}

-- Current navigation
local currentPath = "/"
local pathHistory = {"/"}

-- Load configuration
function loadConfig()
  local file = io.open(CONFIG_FILE, "r")
  if file then
    local content = file:read("*a")
    file:close()
    local success, result = pcall(cjson.decode, content)
    if success then
      config = result
    end
  end
end

-- Save configuration
function saveConfig()
  local file = io.open(CONFIG_FILE, "w")
  if file then
    file:write(cjson.encode(config))
    file:close()
  end
end

-- Layout definition
layout = {
  LinearLayout,
  orientation = "vertical",
  layout_width = "match_parent",
  layout_height = "match_parent",
  {
    LinearLayout,
    orientation = "horizontal",
    layout_width = "match_parent",
    layout_height = "wrap_content",
    padding = "8dp",
    {
      Button,
      id = "backButton",
      text = "Back",
      layout_width = "wrap_content",
      layout_height = "wrap_content"
    },
    {
      TextView,
      id = "currentPathText",
      layout_width = "0dp",
      layout_height = "wrap_content",
      layout_weight = 1,
      gravity = "center_vertical",
      padding = "8dp",
      textSize = "14sp"
    },
    {
      Button,
      id = "exitButton",
      text = "Exit",
      layout_width = "wrap_content",
      layout_height = "wrap_content"
    }
  },
  {
    ListView,
    id = "itemList",
    layout_width = "match_parent",
    layout_height = "0dp",
    layout_weight = 1
  },
  {
    LinearLayout,
    orientation = "horizontal",
    layout_width = "match_parent",
    layout_height = "wrap_content",
    padding = "8dp",
    {
      Button,
      id = "newFolderButton",
      text = "New Folder",
      layout_width = "0dp",
      layout_weight = 1,
      layout_height = "wrap_content"
    },
    {
      Button,
      id = "newMessageButton",
      text = "New Message",
      layout_width = "0dp",
      layout_weight = 1,
      layout_height = "wrap_content"
    }
  }
}

-- Create dialog
local dlg = LuaDialog(this)
dlg.setTitle("Message Manager")
dlg.setView(loadlayout(layout))

-- Function to get current folder content
function getCurrentFolderContent()
  local items = {}
  local currentFolder = config
  
  for folder in currentPath:gmatch("[^/]+") do
    if currentFolder.folders and currentFolder.folders[folder] then
      currentFolder = currentFolder.folders[folder]
    end
  end
  
  if currentFolder.folders then
    for name, _ in pairs(currentFolder.folders) do
      table.insert(items, {
        type = "folder",
        name = name,
        display = "馃搧 "..name
      })
    end
  end
  
  if currentFolder.messages then
    for _, message in ipairs(currentFolder.messages) do
      table.insert(items, {
        type = "message",
        name = message.title,
        content = message.content,
        display = "馃摑 "..message.title
      })
    end
  end
  
  table.sort(items, function(a, b)
    if a.type ~= b.type then
      return a.type == "folder"
    end
    return a.name < b.name
  end)
  
  return items, currentFolder
end

-- Function to update item list
function updateItemList()
  currentPathText.text = currentPath == "/" and "Home" or currentPath
  local items, _ = getCurrentFolderContent()
  local displayItems = {}
  
  for _, item in ipairs(items) do
    table.insert(displayItems, item.display)
  end
  
  itemList.adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, String(displayItems))
end

-- Function to create new folder
function createNewFolder()
  local editDialog = LuaDialog(this)
  editDialog.setTitle("New Folder")
  
  local edit = EditText(this)
  edit.setHint("Folder name")
  editDialog.setView(edit)
  
  editDialog.setPositiveButton("Create", function()
    local folderName = edit.text
    if #folderName > 0 then
      local _, currentFolder = getCurrentFolderContent()
      
      if not currentFolder.folders then
        currentFolder.folders = {}
      end
      
      if not currentFolder.folders[folderName] then
        currentFolder.folders[folderName] = {
          folders = {},
          messages = {}
        }
        saveConfig()
        updateItemList()
      else
        Toast.makeText(this, "Folder already exists", Toast.LENGTH_SHORT).show()
      end
    end
  end)
  
  editDialog.setNegativeButton("Cancel", nil)
  editDialog.show()
end

-- Function to create new message
function createNewMessage()
  local messageDialog = LuaDialog(this)
  messageDialog.setTitle("New Message")
  
  local messageLayout = {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
      EditText,
      id = "titleEdit",
      hint = "Message title",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      singleLine = true
    },
    {
      ScrollView,
      layout_width = "match_parent",
      layout_height = "0dp",
      layout_weight = 1,
      {
        EditText,
        id = "contentEdit",
        hint = "Message content",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        gravity = "top|left",
        background = nil,
        minLines = 10
      }
    }
  }
  
  messageDialog.setView(loadlayout(messageLayout))
  
  local window = messageDialog.getWindow()
  window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
  
  messageDialog.setPositiveButton("Save", function()
    local title = titleEdit.text
    local content = contentEdit.text
    
    if #title > 0 and #content > 0 then
      local _, currentFolder = getCurrentFolderContent()
      
      if not currentFolder.messages then
        currentFolder.messages = {}
      end
      
      table.insert(currentFolder.messages, {
        title = title,
        content = content
      })
      
      saveConfig()
      updateItemList()
    else
      Toast.makeText(this, "Title and content cannot be empty", Toast.LENGTH_SHORT).show()
    end
  end)
  
  messageDialog.setNegativeButton("Cancel", nil)
  messageDialog.show()
end

-- Function to view message
function viewMessage(message)
  local viewDialog = LuaDialog(this)
  viewDialog.setTitle(message.title)
  
  local messageView = {
    ScrollView,
    layout_width = "match_parent",
    layout_height = "match_parent",
    {
      LinearLayout,
      orientation = "vertical",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      padding = "16dp",
      {
        TextView,
        id = "messageContent",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        textIsSelectable = true,
        text = message.content
      }
    }
  }
  
  viewDialog.setView(loadlayout(messageView))
  
  local window = viewDialog.getWindow()
  window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
  
  viewDialog.setPositiveButton("Close", nil)
  viewDialog.show()
end

-- Button handlers
backButton.onClick = function()
  if #pathHistory > 1 then
    table.remove(pathHistory)
    currentPath = pathHistory[#pathHistory]
    updateItemList()
  else
    dlg.dismiss()
  end
end

exitButton.onClick = function()
  dlg.dismiss()
end

newFolderButton.onClick = createNewFolder
newMessageButton.onClick = createNewMessage

-- List item click handler
itemList.onItemClick = function(parent, view, position, id)
  local items = getCurrentFolderContent()
  local item = items[position + 1]
  
  if item.type == "folder" then
    currentPath = currentPath == "/" and "/"..item.name or currentPath.."/"..item.name
    table.insert(pathHistory, currentPath)
    updateItemList()
  else
    viewMessage(item)
  end
end

-- Load initial configuration and update list
loadConfig()
updateItemList()

-- Show dialog
dlg.show()